/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 *		lSettings Class - For simple data saving
 *				v 0.1  -  Attribute Data Saving and Reading Only
 *
 *			Joseph D. Groover, JR
 *
 *		SOURCE CODE UNDER looncraz™ LICENSE
 *		http://looncraz.tripod.com/license.html
*/


#include "lSettings.h"
#include <stdio.h>
#include <TypeConstants.h>
#include <NodeInfo.h>


lSettings :: lSettings(const char*file)
{
	BEntry entry(file, true);
	pref_file.SetTo(&entry);entry.Unset();
	CurFile = file;
}

void lSettings :: ChangeFile(const char*newFile)
{	CurFile = newFile;	
	BEntry entry(CurFile, true);
	pref_file.SetTo(&entry);entry.Unset();
}


bool lSettings :: FileExists()
{	if (pref_file.InitCheck() != B_OK){return false;}
	return(true); 
}

bool lSettings :: CreateFile(const char* file)
{
	if (!strcmp(file,"")){file = CurFile;}
	BString str("touch ");	str << file;
	system(str.String());str.Truncate(0);
	ChangeFile(CurFile);
	return(FileExists()); // checks to see if it was successful
							// should work, will see :-)
}

BBitmap* lSettings :: FileIcon(icon_size which)
{
	BRect lBitmapSize(0,0,15,15);
	if (which == B_LARGE_ICON)lBitmapSize.right += 16;lBitmapSize.bottom += 16;
	
	BBitmap* bmp = new BBitmap(lBitmapSize, B_COLOR_8_BIT);
	BNode node(CurFile);
	BNodeInfo node_info(&node);
	node_info.GetIcon(bmp, which);
	node.Unset();


	return(bmp);
}


// WRITING:
	// I use a cheapo method :-)  Easy though :-)
	// I will probably eventually use real methods..
	// But for now, this is good enough for me.
	// Also gives command line output only when 
	//  needed without any extra code :-)
	
void lSettings :: WriteBool(const char*name, bool value, bool encrypt)
{
	if (strcmp(name,"")) // if not empty (returns true)
	{
		BString val; val << (int)value;
		BString str("addattr -t int ");
		str << "\""<< name << "\" \"" << val << "\" " << CurFile;
		system(str.String());str.Truncate(0);
	}
}

void lSettings :: 	WriteInt32	(const char*name, 	int32 value, 		bool encrypt)
{
	if (strcmp(name,"")) // if not empty (returns true)
	{
		BString val; val << value;
		BString str("addattr -t int32 ");
		str << "\""<< name << "\" \"" << val << "\" " << CurFile;
		system(str.String());str.Truncate(0);
	}
}

void lSettings :: 	WriteString	(const char*name, 	const char*value,	bool encrypt)
{
	if (strcmp(name,"")) // if not empty (returns true)
	{
		BString val; val << value;
		BString str("addattr ");
		str << "\""<< name << "\" \"" << val << "\" " << CurFile;
		system(str.String());str.Truncate(0);
	}
}

void lSettings :: 	WriteRect	(const char*name, 	BRect value, bool encrypt)
{
	if (strcmp(name,"")) // if not empty (returns true)
	{
		const char* app="addattr -t int \"";
		// SPECIAL CASE
		BString str("");
		str << app << name <<".left\" \"" << value.left << "\" \"" << CurFile << "\"\n";
		str << app << name <<".top\" \"" << value.top << "\" \"" << CurFile << "\"\n";
		str << app << name <<".right\" \"" << value.right << "\" \"" << CurFile << "\"\n";
		str << app << name <<".bottom\" \"" << value.bottom << "\" \"" << CurFile << "\"\n";
		system(str.String());str.Truncate(0);
	}
}

void lSettings :: WriteRGB	(const char*name,	rgb_color value, bool encrypt)
{
	BString str(name); str << ".red";
	WriteInt32(str.String(), value.red, encrypt);
	str.RemoveLast(".red");str <<".green";
	WriteInt32(str.String(), value.green, encrypt);
	str.RemoveLast(".green");str <<".blue";
	WriteInt32(str.String(), value.blue, encrypt);
	str.RemoveLast(".blue");str <<".alpha";
	WriteInt32(str.String(), value.alpha, encrypt);
	
	str.Truncate(0);
}

void lSettings :: 	WritePoint	(const char*name, 	BPoint value, bool encrypt)
{
	if (strcmp(name,"")) // if not empty (returns true)
	{
		// SPECIAL CASE
		BString str("");
		str << name << ".x";
		WriteInt32(str.String(), value.x, encrypt);
		str.Truncate(0);
		str << name << ".y";
		WriteInt32(str.String(), value.y, encrypt);
		str.Truncate(0);
		
	}
}


// READING:

bool 			lSettings :: ReadBool	(const char*name, bool encrypted)
{
	return(atoi(r_attr(name).String()));
}

int32 			lSettings :: ReadInt32	(const char*name, bool encrypted)
{
	return(atoi(r_attr(name).String()));
}

const char* 	lSettings :: ReadString	(const char*name, bool encrypted)
{
	return(r_attr(name).String());
}

BRect 			lSettings :: ReadRect	(const char*name, bool encrypted)
{
	// Needs conversion to float

	BRect retVal(0,0,0,0);
	BString str(name);	
	BString str1 = str;
				str1 << ".left";	retVal.left = ReadInt32(str1.String(), encrypted);
	str1 = str;	str1 << ".top";		retVal.top	= ReadInt32(str1.String(), encrypted);
	str1 = str;	str1 << ".right";	retVal.right = ReadInt32(str1.String(), encrypted);
	str1 = str;	str1 << ".bottom";	retVal.bottom = ReadInt32(str1.String(), encrypted);
	str.Truncate(0);	str1.Truncate(0);
	return(retVal);
}

rgb_color lSettings :: ReadRGB(const char*name, bool encrypted=false)
{
	rgb_color color;
	BString str(name); str << ".red";
	color.red 	= ReadInt32(str.String(), encrypted);
	str.RemoveLast(".red"); str << ".green";
	color.green = ReadInt32(str.String(), encrypted);
	str.RemoveLast(".green"); str << ".blue";
	color.blue 	= ReadInt32(str.String(), encrypted);
	str.RemoveLast(".blue"); str << ".alpha";
	color.alpha = ReadInt32(str.String(), encrypted);
	str.Truncate(0);
	return(color);
}

BPoint 			lSettings :: ReadPoint	(const char*name, bool encrypted=false)
{
	// Needs conversion to float
	
	BString str; str << name << ".x";
	int xv = ReadInt32(str.String());
	str.Truncate(0);	str << name << ".y";
	int yv = ReadInt32(str.String());
	str.Truncate(0);
	BPoint ret(xv, yv);
	return(ret);
}

// Special Functions
void	lSettings :: WriteWindowFrame(BRect rect)
{
	WriteRect("l_saved_window_frame",rect);
}

BRect	lSettings :: ReadWindowFrame()
{
	return(ReadRect("l_saved_window_frame"));
}

// so nothing goes awry while developing, these return the originals for now

BString lSettings :: Encrypt(BString encrypt_this)
{
		return(encrypt_this);
}

BString lSettings :: Decrypt(BString decrypt_this)
{
	return(decrypt_this);
}

//  Private method to get information from attributes
BString lSettings :: r_attr(const char*name)
{
	BString Result;
	
	attr_info Prefs_from_File; 
	pref_file.GetAttrInfo( name, &Prefs_from_File );
	switch( Prefs_from_File.type )
	{
		case B_INT32_TYPE:
		{
			int32 PrefBuffer;
			pref_file.ReadAttr( name, Prefs_from_File.type, 0, &PrefBuffer, Prefs_from_File.size );
			Result << PrefBuffer;

			break;
		}
		case B_BOOL_TYPE:
		{
			bool PrefBuffer;	
			pref_file.ReadAttr( name, Prefs_from_File.type, 0, &PrefBuffer, Prefs_from_File.size );
			Result << (int)PrefBuffer;

			break;
		}
		case B_STRING_TYPE:
		{
			char text[256];
			pref_file.ReadAttr(name, B_STRING_TYPE, 0, text, Prefs_from_File.size);
			Result << text;
			break;

			break;
		}
	}	

	
	
	
	return(Result);
}

